#include "ParamBool.h"
#include "ParamFloat.h"
namespace ffgl
{
ParamBool::ParamBool( const std::string& name, bool defaultValue ) :
	ParamBool( name, ParameterType::FF_TYPE_BOOLEAN, name, defaultValue )
{
}

ParamBool::ParamBool( const std::string& name, const std::string& uniformName, bool defaultValue ) :
	ParamBool( name, ParameterType::FF_TYPE_BOOLEAN, uniformName, defaultValue )
{
}

ParamBool::ParamBool( const std::string& name, ParameterType type, bool defaultValue ) :
	ParamBool( name, type, name, defaultValue )
{
}

ParamBool::ParamBool( const std::string& name, ParameterType type, const std::string& uniformName, bool defaultValue ) :
	ParamFloat( name, type, uniformName, defaultValue ? 1.0f : 0.0f )
{
	// Underlying float values: True is 1.0, false is 0.0
}

const std::string& ParamBool::GetDisplay() const
{
	static const std::string displayTrue( "True" );
	static const std::string displayFalse( "False" );
	return GetBoolValue() ? displayTrue : displayFalse;
}

bool ParamBool::GetBoolDefault() const
{
	// Non-zero is true
	return defaultValue != 0.0f;
}

bool ParamBool::GetBoolValue() const
{
	// Non-zero is true
	return value != 0.0f;
}

void ParamBool::SetBoolValue( const bool boolValue )
{
	// Underlying float values: True is 1.0, false is 0.0
	value = boolValue ? 1.0f : 0.0f;
}

}//End namespace ffgl
