#include "ParamBuffer.h"

namespace ffgl
{

ParamBuffer::ParamBuffer( std::string& name, size_t bufferSize, ParameterUsage usage ) :
	ParamBase( name, ParameterType::FF_TYPE_BUFFER ),
	usage( usage ),
	data( bufferSize )
{
}

ParamBuffer::~ParamBuffer()
{
	// Note: data vector will dispose of itself
}

void ParamBuffer::Reset()
{
	for (auto elem : data)
	{
		elem = 0.0f;
	}
}

const std::string& ParamBuffer::GetDisplay() const
{
	static const std::string displayString( "[BUFFER]" );
	return displayString;
}

FFMixed ParamBuffer::GetMixedDefault() const
{
	// There is no default value to return for a buffer
	FFMixed retval;
	retval.UIntValue = FF_FAIL;
	return retval;
}

FFMixed ParamBuffer::GetMixedValue() const
{
	// There is no single value to return for a buffer, but we can return the address of the buffer
	FFMixed retval;
	retval.UIntValue = FF_FAIL;

	// Fail if there is no buffer
	if( data.size() == 0 )
		return retval;

	retval.PointerValue = (void*)data.data();
	return retval;
}

FFResult ParamBuffer::SetMixedValue( const FFMixed mixed )
{
	// Set every element of the buffer with the float value
	for( auto elem : data )
	{
		elem = mixed.FloatValue;
	}

	return FF_SUCCESS;
}

ParameterUsage ParamBuffer::GetUsage() const
{
	return usage;
}

}//End namespace ffgl
