// spxlSubAdd Effect Plugin
// More or less the same result as the AddSubtract sample plugin, with extra parameters for testing purposes.
// The "multiplier" parameter is used to scale the R,G,B addition/subtraction.

// 2019-08-12 ffgl@subpixels.com
// - Created, based on AddSubbtract sample code

#include "spxlSubAdd.h"
#include "ffgl/ParamBool.h"
#include "ffgl/ParamBuffer.h"
#include "ffgl/ParamFloat.h"
#include "ffgl/ParamInteger.h"
#include "ffgl/ParamOption.h"
#include "ffgl/ParamRange.h"
#include "ffglquickstart/ParamEvent.h"
#include "ffglquickstart/ParamFFT.h"
#include "ffglquickstart/ParamTrigger.h"


using namespace ffgl;
using namespace ffglex;
using namespace ffglqs;

namespace spxl
{
namespace ffgl
{
namespace effect
{

static ::ffgl::PluginInfo pluginInfo(
	PluginFactory< SubAdd >,// Create method
#ifdef _DEBUG
	"?e01",               // Plugin unique ID of maximum length 4.
	"?SubAdd",            // Plugin name
#else
	"_e01",               // Plugin unique ID of maximum length 4.
	"_SubAdd",            // Plugin name
#endif
	2,                    // API major version number
	1,                    // API minor version number
	1,                    // Plugin major version number
	0,                    // Plugin minor version number
	PluginType::FF_EFFECT,// Plugin type

	"Subtract and Add colours",  // Plugin description
	"Resolume FFGL Example (mod by ffgl@subpixels.com)" // About
);

static const char _fragmentShaderCode[] = R"(
void main()
{
	vec4 color = texture( inputTexture, i_uv );
	//The InputTexture contains premultiplied colors, so we need to unpremultiply first to apply our effect on straight colors.
	if( color.a > 0.0 )
		color.rgb /= color.a;

	color.rgb += (Brightness * 2. - 1.) * multiplier ;

	//The plugin has to output premultiplied colors, this is how we're premultiplying our straight color while also
	//ensuring we aren't going out of the LDR the video engine is working in.
	color.rgb = clamp( color.rgb * color.a, vec3( 0.0 ), vec3( color.a ) );
	fragColor = color;
}
)";

SubAdd::SubAdd()
{
	//We declare that this plugin has a Brightness parameter which is a RGB param.
	//The name here must match the one you declared in your fragment shader.
	AddRGBColorParam( "Brightness" );

	AddHueColorParam( "MyColor1" );
	AddHueColorParam( "MyColor2" );

	// Test Parameters
	AddParam( new ParamFloat( "NameOnly" ) );
	AddParam( new ParamFloat( "NameValue", 0.1f ) );
	AddParam( new ParamFloat( "NameTypeValue", ParameterType::FF_TYPE_BRIGHTNESS, 0.9f ) );
	AddParam( new ParamFloat( "Multiplier", ParameterType::FF_TYPE_STANDARD, "multiplier", 1.0f ) );
	AddParam( new ParamBool( "TestBool" ) );
	AddParam( new ParamEvent( "TestEvent" ) );
	AddFFTParam( new ParamFFT( "TestFFT", 8 ) );

	ParamOption* paramOption = new ParamOption( "TestOption" );
	paramOption->AddOption( { "Test1", 11.1f } );
	paramOption->AddOption( { "Test2", 22.2f } );
	paramOption->AddOption( { "Test3", 33.3f } );
	AddParam( paramOption );

	AddParam( new ParamRange( "TestRange", 15.0f, { 0.0f, 100.0f } ) );
	AddParam( new ParamInteger( "TestInteger", 64, { -1920.0f, 1920.0f } ) );
	AddParam( new ParamText( "TestText" ) );
	AddParam( new ParamTrigger( "TestTrigger" ) );

	SetFragmentShader( _fragmentShaderCode );

	SetTimeSupported( true );
}

SubAdd::~SubAdd()
{
}

}// namespace effect
}// namespace ffgl
}// namespace spxl
